# Use LS colors for ZSH completion
zstyle ':completion:*' list-colors ${(s.:.)LS_COLORS}
zstyle ':completion:*' menu select

if [ -n "${commands[fzf-share]}" ]; then
  source "$(fzf-share)/key-bindings.zsh"
  source "$(fzf-share)/completion.zsh"
fi

# create a zkbd compatible hash;
# to add other keys to this hash, see: man 5 terminfo
typeset -g -A key

# We need this here since it's used below
if command -v fdfind &> /dev/null; then
    alias fd="fdfind"
fi

# Use vi-mode
ZVM_VI_INSERT_ESCAPE_BINDKEY=jj

# Finally, make sure the terminal is in application mode, when zle is
# active. Only then are the values from $terminfo valid.
if (( ${+terminfo[smkx]} && ${+terminfo[rmkx]} )); then
	autoload -Uz add-zle-hook-widget
	function zle_application_mode_start { echoti smkx }
	function zle_application_mode_stop { echoti rmkx }
	add-zle-hook-widget -Uz zle-line-init zle_application_mode_start
	add-zle-hook-widget -Uz zle-line-finish zle_application_mode_stop
fi

function cd_to() {
    setopt localoptions pipefail no_aliases 2> /dev/null
    local dir=$1
    if [[ -z "$dir" ]]; then
        zle redisplay
        return 0
    fi
    zle push-line # Clear buffer. Auto-restored on next prompt.
    cd $dir
    zle accept-line
    local ret=$?
    unset dir # ensure this doesn't end up appearing in prompt expansion
    zle reset-prompt
}

function find_local() {
    fd . -t $1 | fzy -l 20
}

function find_global() {
    file_type=$1
    FZY_CACHE=~/.cache/fzy_paths_$file_type
    if [ -f $FZY_CACHE ]; then
        if [ $(stat -c %Y $FZY_CACHE) -lt $(date +%s -d '-1 day') ]; then
            rm $FZY_CACHE
        fi
    fi
    if [ ! -f $FZY_CACHE ]; then
        PATHS="$(/usr/bin/git --git-dir=$HOME/.dotfiles/ --work-tree=$HOME ls-files)"
        if [ "$file_type" = "d" ]; then
            PATHS="$(echo -e $PATHS | xargs -n 1 dirname | uniq | grep -v '^\.$' | parallel echo ~/{})"
        else
            PATHS="$(echo -e $PATHS | parallel echo ~/{})"
        fi
        search_dirs=(~/Workspace/ ~/Containers/ /home/server/Storage/Thomas/)
        for i in "$search_dirs[@]"; do
            if [ -d $i ]; then
                PATHS="$PATHS\n$i"
                PATHS+="\n$(fd . $i -t $file_type -d 8)"
            fi
        done
        echo -e "$PATHS" > $FZY_CACHE
    fi
    file="$(cat $FZY_CACHE | fzy -l 20)"
    echo "$file" | cat - $FZY_CACHE | awk '!x[$0]++' > /tmp/fzy_paths_$file_type && mv /tmp/fzy_paths_$file_type $FZY_CACHE
    echo "$file"
}

function open_path() {
    file_type=$1
    search_fn=$2
    selection="$($search_fn $file_type)"
    if [ "$selection" = "" ]; then
        zle reset-prompt
        return
    fi
    if [ "$LBUFFER" = "" ]; then
        if [ "$file_type" = "d" ]; then
            cd_to "$selection"
        else
            selection=$(realpath "$selection")
            cd_to "$(dirname \"$selection\")"
            nvim "$selection"
        fi
    else
        emulate -L zsh
        zle -I
        LBUFFER="${LBUFFER}\"$selection\""
        zle reset-prompt
    fi
}

function zvm_after_init() {
    key[Home]="${terminfo[khome]}"
    key[End]="${terminfo[kend]}"
    key[Insert]="${terminfo[kich1]}"
    key[Backspace]="${terminfo[kbs]}"
    key[Delete]="${terminfo[kdch1]}"
    key[Up]="${terminfo[kcuu1]}"
    key[Down]="${terminfo[kcud1]}"
    key[Left]="${terminfo[kcub1]}"
    key[Right]="${terminfo[kcuf1]}"
    key[PageUp]="${terminfo[kpp]}"
    key[PageDown]="${terminfo[knp]}"
    key[Shift-Tab]="${terminfo[kcbt]}"

    # setup key accordingly
    [[ -n "${key[Home]}"      ]] && bindkey -- "${key[Home]}"       beginning-of-line
    [[ -n "${key[End]}"       ]] && bindkey -- "${key[End]}"        end-of-line
    [[ -n "${key[Insert]}"    ]] && bindkey -- "${key[Insert]}"     overwrite-mode
    [[ -n "${key[Backspace]}" ]] && bindkey -- "${key[Backspace]}"  backward-delete-char
    [[ -n "${key[Delete]}"    ]] && bindkey -- "${key[Delete]}"     delete-char
    [[ -n "${key[Up]}"        ]] && bindkey -- "${key[Up]}"         up-line-or-history
    [[ -n "${key[Down]}"      ]] && bindkey -- "${key[Down]}"       down-line-or-history
    [[ -n "${key[Left]}"      ]] && bindkey -- "${key[Left]}"       backward-char
    [[ -n "${key[Right]}"     ]] && bindkey -- "${key[Right]}"      forward-char
    [[ -n "${key[PageUp]}"    ]] && bindkey -- "${key[PageUp]}"     beginning-of-buffer-or-history
    [[ -n "${key[PageDown]}"  ]] && bindkey -- "${key[PageDown]}"   end-of-buffer-or-history
    [[ -n "${key[Shift-Tab]}" ]] && bindkey -- "${key[Shift-Tab]}"  reverse-menu-complete

    autoload -Uz up-line-or-beginning-search down-line-or-beginning-search
    zle -N up-line-or-beginning-search
    zle -N down-line-or-beginning-search
    [[ -n "${key[Up]}"   ]] && bindkey -- "${key[Up]}"   up-line-or-beginning-search
    [[ -n "${key[Down]}" ]] && bindkey -- "${key[Down]}" down-line-or-beginning-search


    if [ -f "/usr/share/doc/pkgfile/command-not-found.zsh" ]; then
        source /usr/share/doc/pkgfile/command-not-found.zsh
    fi

    #FZF
    if [ -f "/usr/share/doc/fzf/examples/key-bindings.zsh" ]; then
        source /usr/share/doc/fzf/examples/key-bindings.zsh
        source /usr/share/doc/fzf/examples/completion.zsh
    elif [ -f "/usr/share/fzf/key-bindings.zsh" ]; then
        source /usr/share/fzf/key-bindings.zsh
        source  /usr/share/fzf/completion.zsh
    fi

    if [ -f "/usr/share/zsh/plugins/zsh-autosuggestions/zsh-autosuggestions.zsh" ]; then
        source /usr/share/zsh/plugins/zsh-autosuggestions/zsh-autosuggestions.zsh
    fi

    function open_global_path_d() { open_path d find_global }
    function open_global_path_f() { open_path f find_global }
    function open_local() { open_path f find_local }

    zle -N push_local_files
    zle -N find_local_directories

    function register_key() {
        zle -N $2
        bindkey "^$1" $2
        bindkey -M emacs "^$1" $2
        bindkey -M vicmd "^$1" $2
        bindkey -M viins "^$1" $2
    }

    register_key "T" open_global_path_d
    register_key "E" open_global_path_f
    register_key "W" open_local

    if [ "$SSH_SESSION_ID" != "" ]; then
        mkdir -p /tmp/ssh_sessions
        echo $$ > /tmp/ssh_sessions/"$SSH_SESSION_ID"
    fi
    if [ "$REMOTE_PATH" != "" ]; then
        if [ -d "$REMOTE_PATH" ]; then
            cd "$REMOTE_PATH"
        fi
    elif [ "$PREVIOUS_SESSION_ID" != "" ]; then
        cd "$(readlink -e /proc/$(cat /tmp/ssh_sessions/$PREVIOUS_SESSION_ID)/cwd)"
    fi
}

zstyle ':completion:*' matcher-list '' 'm:{a-zA-Z}={A-Za-z}' 'r:|=*' 'l:|=* r:|=*'
zstyle ':completion::complete:*' gain-privileges 1
zstyle ':completion:*' rehash true


autoload -Uz compinit
compinit -d "$XDG_CACHE_HOME"/zsh/zcompdump-"$ZSH_VERSION"


export HISTFILE="$XDG_STATE_HOME"/zsh/history
export HISTSIZE=10000000
export SAVEHIST=10000000

unsetopt share_history

BROWSER=/usr/bin/firefox

# Add bash aliases.
if [ -f ~/.bash_aliases ]; then
    source ~/.bash_aliases
fi

# GPULab
export GPULAB_CERT="$XDG_DATA_HOME/gpulab.pem"
export GPULAB_DEV='False'

# Smartcd
export SMARTCD_QUIET=1
if [ -f ~/.smartcd_config ]; then
    source ~/.smartcd_config
fi

export GCC_COLORS='error=01;31:warning=01;35:note=01;36:caret=01;32:locus=01:quote=01'
export EDITOR=nvim
export LANG=en_US.UTF-8
export ARCHFLAGS="-arch x86_64"
export SSH_KEY_PATH="~/.ssh/rsa_id"
export PYTHONDONTWRITEBYTECODE=1
export TERM="screen-256color"
export PATH=$HOME/.local/bin:$HOME/.cargo/bin:$PATH
export GOPATH=/tmp/go

if command -v micromamba &> /dev/null; then
    eval "$(micromamba shell hook --shell zsh)"
    export MAMBA_ROOT_PREFIX="$HOME/.micromamba";
fi

autoload -Uz add-zsh-hook

eval "$(zoxide init zsh --cmd j)"

function xterm_title_precmd () {
    print -Pn -- '\e]2;%n@%m %~\a'
}

function xterm_title_preexec () {
	print -Pn -- '\e]2;' && print -n -- "${(q)1}\a"
}


if [[ "$TERM" == (Eterm*|alacritty*|aterm*|gnome*|konsole*|kterm*|putty*|rxvt*|screen*|tmux*|xterm*) ]]; then
	add-zsh-hook -Uz precmd xterm_title_precmd
	add-zsh-hook -Uz preexec xterm_title_preexec
fi


###########################################
# Aliases
###########################################

if [[ "$(hostname)" == "mallorea" ]]; then
    alias lf="/usr/bin/lf -last-dir-path /tmp/lf_dir && cd \"\$(cat /tmp/lf_dir 2> /dev/null || pwd)\""
else
    alias lf="cd \"\$(/usr/bin/lf -print-last-dir)\""
fi

function o () {
	xdg-open "$1" &!
}

function run_waypipe() {
    SSH_SESSION_ID=$RANDOM
    REMOTE_PATH=$(echo $(pwd) | sed "s|/home/[a-z]*|\$HOME|g")
    SSH_SESSION_ID=$SSH_SESSION_ID waypipe ssh -t $1 env REMOTE_PATH=\"$REMOTE_PATH\" SSH_SESSION_ID=$SSH_SESSION_ID "zsh --login"
}

function git() {
    if [ "$HOME" = "$(pwd)" ]; then
        /usr/bin/git --git-dir="$HOME/.dotfiles/" --work-tree="$HOME" "$@"
    else
        /usr/bin/git "$@"
    fi
}
